-- queenboard.e
--
-- Ben Lin 5/98
--
-- n Queens Problem Blackboard, Fast Edition
--
-- Should be more efficient - the nth queen must go in the nth row,
-- eliminating the need for any row observers. Changed local observers to
-- global observer. (Removed all `FULL' observers).

class QUEENBOARD
	-- blackboard for solving the n queens problem
inherit BLACKBOARD
redefine display
creation make
feature
	size : INTEGER
		-- size of the board = number of queens to place
	board : ARRAY2[INTEGER]
		-- `size' x `size' array of integers represents the board
		-- -1 = "bad" mark (no queen may go here)
		--  0 = empty square
		--  1 = queen
	queen_count : INTEGER
		-- number of queens placed so far

	queen_loc : ARRAY[INTEGER]
		-- locations of the queens placed so far

	add_observer : OBSERVER_LIST
		-- notified every time a queen is placed

	make(board_size : INTEGER) is
		-- make a n queens blackboard of size `board_size'
	local
		i : INTEGER
		j : INTEGER
		fill : FILLER
	do
		!!command_history.make
		size := board_size
		!!board.make(1, size, 1, size)
		!!queen_loc.make(1, size)
		!!add_observer.make
		!!fill.make
		add_observer.add_last(fill)
		queen_count := 0
	end

	finished : BOOLEAN is
		-- finished if `size' queens have been placed and the blackboard
		-- is still alive
	do
		Result := queen_count = size and alive
	end

	get_open : like command_history is
		-- returns a list of all possible `add_queen' commands
	local
		aq : ADD_QUEEN
		i : INTEGER
	do
		!!Result.make
		if alive and queen_count < size then
			from i := 1
			until i > size
			loop
				if board.item(1 + queen_count, i) = 0 then
					!!aq.make(Current, queen_count + 1, i)
					Result.add_last(aq)
				end
				i := i + 1
			end
		end

	end

	add_queen(x, y : INTEGER) is
		-- place a queen on (x, y)
	require
		board.item(x, y) = 0
	do
		board.put(1, x, y)
		queen_count := queen_count + 1
		queen_loc.put(y, queen_count)
	end

	mark_bad(x, y : INTEGER) is
		-- mark the square (x, y)
	require
		board.item(x, y) = 0
	do
		board.put(-1, x, y)
	end

	remove_queen(x, y : INTEGER) is
		-- remove the queen on (x, y)
	require
		board.item(x, y) = 1
	do
		board.put(0, x, y)
		queen_count := queen_count - 1
	end

	unmark_bad(x, y : INTEGER) is
		-- remove the mark on (x, y)
	require
		board.item(x, y) = -1
	do
		board.put(0, x, y)
	end

	display is
		-- show the chessboard, with queens and marks
	local
		i, j : integer
	do
		from i := 1
		until i > size
		loop
			from j := 1
			until j > size
			loop
				inspect board.item(i, j)
				when 1 then
					io.put_character('Q')
				when 0 then
					io.put_character('.')
				when -1 then
					io.put_character('x')
				else
					io.put_character('?')
				end
				j := j + 1
			end

			io.put_character('%N')
			i := i + 1
		end
		
	end

end 

