-- Logic problem - "In The Throws"
-- from "Logic Problems" No. 15 (by A.J. Duncum)
-- Ben Lin 26/5/98
--
-- See panic.e for another example

class THROWS
creation make
feature
	pb : LP_BOARD
	size : INTEGER is 5
	sub_count : INTEGER is 4

	-- The 4 categories are:
	-- 1 = Order thrown (2nd, 5th, 7th, 10th, 12th)
	-- 2 = Nationality (East German, Hungarian, Polish, Russian, West German)
	-- 3 = Distance (77.45m, 79.83m, 80.62m, 81.79m, 82.18m)
	-- 4 = Round in which best throw occurred (2nd, 3rd, 4th, 5th, 6th)

	--ss : DFS_SEARCH
	ss : I_SEARCH

	make is
	do
		!!pb.make(sub_count, size)
		setup_clues
		!!ss
		ss.doit(pb)
		if not pb.finished then
			io.put_string("No solution! :(%N")
		else
			io.put_string("Solution found:%N")
			pb.display
		end
	end

	setup_clues is
	do
		-- Observers must come first

		larger_than(2, 4, 2, 3, 3)
			-- 1. Pole beaten by the Russian

		connect(2, 4, 3, 2, 2, 4, 4, 1)
			-- 1. Pole threw in his best in the 3rd round,
			-- iff the Russian did his best in the 2nd

		connect(2, 4, 3, 3, 2, 4, 4, 2)
			-- 1. Pole threw in his best in the 4th round,
			-- iff the Russian did his best in the 3rd

		connect(2, 4, 3, 4, 2, 4, 4, 3)
			-- 1. Pole threw in his best in the 5th round,
			-- iff the Russian did his best in the 4th

		connect(2, 4, 3, 5, 2, 4, 4, 4)
			-- 1. Pole threw in his best in the 6th round,
			-- iff the Russian did his best in the 5th

		connect(1, 2, 2, 1, 1, 2, 3, 5)
			-- 6. If East German was 5th to throw, then West German was 7th
			-- and vice versa

		connect(1, 2, 3, 1, 1, 2, 2, 5)
			-- 6. If East German was 7th to throw, then West German was 5th
			-- and vice versa

		connect(1, 2, 4, 1, 1, 2, 5, 5)
			-- 6. If East German was 10th to throw, then West German was 12th
			-- and vice versa

		connect(1, 2, 5, 1, 1, 2, 4, 5)
			-- 6. If East German was 12th to throw, then West German was 10th
			-- and vice versa

		larger_than(2, 1, 2, 5, 3)
			-- 6. ..the East German beat the West German

		-- Ticks and crosses

		tick(1, 2, 3, 3)
			-- 1. Polish was 7th to throw in each round

		cross(2, 4, 3, 1)
			-- 1. ...and he couldn't have got his best in the 2nd round

		tick(3, 4, 2, 3)
			-- 2. Someone had a best of 79.83m in his 4th round

		tick(1, 4, 5, 5)
			-- 3. The one who threw 12th got his best in the 6th round

		cross(1, 3, 5, 1)
			-- 3. ...and he didn't finish 5th

		tick(2, 3, 2, 4)
			-- 4. Hungarian won the silver medal

		cross(2, 4, 2, 2)
		cross(2, 4, 2, 4)
			-- 4. ...in an even numbered round

		cross(1, 2, 1, 2)
			-- 4. ...and he wasn't the 2nd to throw in each round

		tick(1, 3, 4, 3)
			-- 5. The 10th-to-throw's best was 80.62m

		cross(1, 2, 1, 5)
		cross(1, 2, 1, 1)
			-- 6. 2nd to throw was not East or West German
	end

	-- should go in lp_board?
	connect(i1, j1, x1, y1, i2, j2, x2, y2 : INTEGER) is
	local
		tl : TICK_LINK
		cl : CROSS_LINK
	do
		-- tick-link and cross-link both ways
		!!tl.make(i2, j2, x2, y2)
		pb.tick_observer.item(i1, j1, x1, y1).add_last(tl)
		!!cl.make(i2, j2, x2, y2)
		pb.cross_observer.item(i1, j1, x1, y1).add_last(cl)
		!!tl.make(i1, j1, x1, y1)
		pb.tick_observer.item(i2, j2, x2, y2).add_last(tl)
		!!cl.make(i1, j1, x1, y1)
		pb.cross_observer.item(i2, j2, x2, y2).add_last(cl)
	end
	
	larger_than(i1, x1, i2, x2, j : INTEGER) is
	local
		y : INTEGER
		larger : LARGER_THAN
		smaller : SMALLER_THAN
	do
		from y := 1
		until y > size
		loop
			-- for some reason, this problem can be solved even if
			-- one pair is commented out (eg use larger_than only)
			!!larger.make(i1, x1, i2, x2, j)
			pb.tick_observer.item(i1, j, x1, y).add_last(larger)
			pb.cross_observer.item(i1, j, x1, y).add_last(larger)
			!!smaller.make(i2, x2, i1, x1, j)
			pb.tick_observer.item(i2, j, x2, y).add_last(smaller)
			pb.cross_observer.item(i2, j, x2, y).add_last(smaller)
			y := y + 1
		end
	end

	tick(i, j, x, y : INTEGER) is
	local
		at : ADD_TICK
	do
		!!at.make(pb, i, j, x, y)
		do_initial_command(at)
	end

	cross(i, j, x, y : INTEGER) is
	local
		ac : ADD_CROSS
	do
		!!ac.make(pb, i, j, x, y)
		do_initial_command(ac)
	end

	do_initial_command(c : COMMAND) is
	do
		if c.finished then
			io.put_string("warning: ")
			c.display
			io.put_string("already executed%N")
		elseif not c.can_execute then
			io.put_string("warning: ")
			c.display
			io.put_string(" => no solution%N")
		end
		c.execute
	end
end

