-- multicommand.e
--
-- Ben Lin 6/98
--
-- This allows execution of a list of several commands before their forced moves
-- are found and executed (normally, as soon as a command is executed, it's
-- forced moves are as well) In a sense, these commands are done at once.

class MULTICOMMAND
	-- Allows execution of several commands before any of their forced moves
	-- are executed. Inherits from COMMAND_LIST as well - use its features
	-- to manipulate the list of commands.
inherit
	COMMAND
		undefine
			fill_tagged_out_memory,
			copy,
			is_equal
		redefine
			display
		end;
	COMMAND_LIST
		rename
			make as make_list
		end
creation make
feature
	make(blackboard : BLACKBOARD) is
		-- Initialize a list of commands that will affect blackboard
	do
		bb := blackboard
		make_list
	end

	plain_execute is
	require
		personal_history = Void
	local
		i : INTEGER
	do
		!!personal_history.make
		from i := lower
		until i > upper or bb.dead
		loop
			if bb.alive then
				if item(i).can_execute then
					personal_history.add_last(item(i))
					item(i).plain_execute
				elseif not item(i).finished then
					contradiction := true
					bb.suicide
				end
			end

			i := i + 1
		end
	end

	plain_unexecute is
	require
		personal_history /= Void

	local
		i : INTEGER
	do
		if contradiction then
			contradiction := false
			bb.rebirth
		end

		from i := upper
		until i < lower
		loop
			personal_history.item(i).plain_unexecute
			i := i - 1
		end
		personal_history := Void
	end

	append_forced is
	require
		personal_history /= Void
	local
		i : INTEGER
	do
		from i := lower
		until i > upper
		loop
			item(i).put_forced_move(forced_move)
			personal_history.item(i).append_forced
			i := i + 1
		end
	end

	can_execute : BOOLEAN is true
		-- can always execute a multicommand

	finished : BOOLEAN is false
		-- can't tell if it's been done

	display is
	local
		i : INTEGER
	do
		io.put_string("Multicommand:%N")
		from i := lower
		until i > upper
		loop
			io.put_integer(i)
			io.put_string(": ")
			item(i).display
			i := i + 1
		end
	end

feature {NONE}

	personal_history : COMMAND_LIST
end
